@@ -29,6 +29,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/mman.h>
 #include <fts.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -47,14 +49,28 @@
 #include "kernel.h"
 
 
+static void find_conflicting_xfree86_libraries(Options *,
+                                               const char *,
+                                               FileList *);
+
+static void find_conflicting_xfree86_libraries_fullpath(Options *op,
+                                                        const char *,
+                                                        FileList *l);
+
+static void find_conflicting_opengl_libraries(Options *,
+                                              const char *,
+                                              FileList *);
 
-static void find_existing_files(Package *p, FileList *l, unsigned int);
 static void find_conflicting_kernel_modules(Options *op,
-                                            Package *p, FileList *l);
+                                            Package *p,
+                                            FileList *l);
+
+static void find_existing_files(Package *p, FileList *l, uint64_t);
+
+static void condense_file_list(Package *p, FileList *l);
 
 static void add_command (CommandList *c, int cmd, ...);
 
-static void find_matches (const char*, const char*, FileList*, const int);
 static void add_file_to_list(const char*, const char*, FileList*);
 
 static void append_to_rpm_file_list(Options *op, Command *c);
@@ -79,7 +95,7 @@ CommandList *build_command_list(Options *op, Package *p)
     FileList *l;
     CommandList *c;
     int i, cmd;
-    unsigned int installable_files;
+    uint64_t installable_files;
     char *tmp;
 
     installable_files = get_installable_file_mask(op);
@@ -87,41 +103,77 @@ CommandList *build_command_list(Options *op, Package *p)
     l = (FileList *) nvalloc(sizeof(FileList));
     c = (CommandList *) nvalloc(sizeof(CommandList));
 
-    /* find any possible conflicting libraries */
+    /* find any possibly conflicting libraries and/or modules */
     
     if (!op->kernel_module_only) {
+        /*
+         * Note that searching the various paths may produce duplicate
+         * entries for conflicting files; this is OK because we will take
+         * care of these duplicates in condense_file_list().
+         */
 
-        find_conflicting_xfree86_libraries
-            (op, DEFAULT_XFREE86_INSTALLATION_PREFIX, l);
-    
-        if (strcmp(DEFAULT_XFREE86_INSTALLATION_PREFIX,
-                   op->xfree86_prefix) != 0)
-            find_conflicting_xfree86_libraries(op, op->xfree86_prefix, l);
-    
-        find_conflicting_opengl_libraries
-            (op, DEFAULT_OPENGL_INSTALLATION_PREFIX, l);
-    
-        if (strcmp(DEFAULT_OPENGL_INSTALLATION_PREFIX, op->opengl_prefix) != 0)
-            find_conflicting_opengl_libraries(op, op->opengl_prefix, l);
+        ui_status_begin(op, "Searching for conflicting X files:", "Searching");
+
+        ui_status_update(op, 0.16f, DEFAULT_X_PREFIX);
+        find_conflicting_xfree86_libraries(op, DEFAULT_X_PREFIX, l);
+        ui_status_update(op, 0.32f, XORG7_DEFAULT_X_PREFIX);
+        find_conflicting_xfree86_libraries(op, XORG7_DEFAULT_X_PREFIX, l);
+        ui_status_update(op, 0.48f, op->x_prefix);
+        find_conflicting_xfree86_libraries(op, op->x_prefix, l);
+
+        ui_status_update(op, 0.64f, op->x_module_path);
+        find_conflicting_xfree86_libraries_fullpath(op, op->x_module_path, l);
+        ui_status_update(op, 0.80f, op->x_library_path);
+        find_conflicting_xfree86_libraries_fullpath(op, op->x_library_path, l);
+
+        ui_status_end(op, "done.");
+
+        ui_status_begin(op, "Searching for conflicting OpenGL files:", "Searching");
+
+        ui_status_update(op, 0.20f, DEFAULT_X_PREFIX);
+        find_conflicting_opengl_libraries(op, DEFAULT_X_PREFIX, l);
+        ui_status_update(op, 0.40f, op->x_prefix);
+        find_conflicting_opengl_libraries(op, op->x_prefix, l);
+        ui_status_update(op, 0.60f, DEFAULT_OPENGL_PREFIX);
+        find_conflicting_opengl_libraries(op, DEFAULT_OPENGL_PREFIX, l);
+        ui_status_update(op, 0.80f, op->opengl_prefix);
+        find_conflicting_opengl_libraries(op, op->opengl_prefix, l);
+
+        ui_status_end(op, "done.");
 
 #if defined(NV_X86_64)
-        if (op->compat32_prefix != NULL) {
-            char *prefix = nvstrcat(op->compat32_prefix,
-                                    DEFAULT_OPENGL_INSTALLATION_PREFIX, NULL);
+        if (op->compat32_chroot != NULL) {
+            char *prefix;
+
+            ui_status_begin(op, "Searching for conflicting compat32 files:", "Searching");
+
+            prefix = nvstrcat(op->compat32_chroot, DEFAULT_X_PREFIX, NULL);
+            ui_status_update(op, 0.20f, prefix);
             find_conflicting_opengl_libraries(op, prefix, l);
             nvfree(prefix);
 
-            if (strcmp(DEFAULT_OPENGL_INSTALLATION_PREFIX,
-                op->opengl_prefix) != 0) {
-                prefix = nvstrcat(op->compat32_prefix, op->opengl_prefix, NULL);
-                find_conflicting_opengl_libraries(op, prefix, l);
-                nvfree(prefix);
-            }
+            prefix = nvstrcat(op->compat32_chroot, op->x_prefix, NULL);
+            ui_status_update(op, 0.40f, prefix);
+            find_conflicting_opengl_libraries(op, prefix, l);
+            nvfree(prefix);
+
+            prefix = nvstrcat(op->compat32_chroot, DEFAULT_OPENGL_PREFIX, NULL);
+            ui_status_update(op, 0.60f, prefix);
+            find_conflicting_opengl_libraries(op, prefix, l);
+            nvfree(prefix);
+
+            prefix = nvstrcat(op->compat32_chroot, op->compat32_prefix, NULL);
+            ui_status_update(op, 0.80f, prefix);
+            find_conflicting_opengl_libraries(op, prefix, l);
+            nvfree(prefix);
+
+            ui_status_end(op, "done.");
         }
 #endif /* NV_X86_64 */
     }
     
-    find_conflicting_kernel_modules(op, p, l);
+    if (!op->no_kernel_module)
+        find_conflicting_kernel_modules(op, p, l);
     
     /*
      * find any existing files that clash with what we're going to
@@ -132,7 +184,7 @@ CommandList *build_command_list(Options *op, Package *p)
     
     /* condense the file list */
 
-    condense_file_list(l);
+    condense_file_list(p, l);
     
     /* check the conflicting file list for any installed files */
 
@@ -159,22 +211,60 @@ CommandList *build_command_list(Options *op, Package *p)
     /* Add all the installable files to the list */
     
     for (i = 0; i < p->num_entries; i++) {
+        /*
+         * Install first, then run execstack. This sets the selinux context on
+         * the installed file in the target filesystem, which is essentially
+         * guaranteed to support selinux attributes if selinux is enabled.
+         * However, the temporary filesystem containing the uninstalled file
+         * may be on a filesystem that doesn't support selinux attributes,
+         * such as NFS.
+         *
+         * Since execstack also changes the file on disk, we need to run it
+         * after the file is installed but before we compute and log its CRC in
+         * the backup log. To achieve that, we tell INTALL_CMD to run execstack
+         * as a post-install step.
+         *
+         * See bugs 530083 and 611327
+         */
+        if (op->selinux_enabled &&
+            (op->utils[EXECSTACK] != NULL) &&
+            ((p->entries[i].flags & FILE_TYPE_SHARED_LIB) ||
+             (p->entries[i].flags & FILE_TYPE_XMODULE_SHARED_LIB))) {
+            tmp = nvstrcat(op->utils[EXECSTACK], " -c ",
+                           p->entries[i].dst, NULL);
+        } else {
+            tmp = NULL;
+        }
+
         if (p->entries[i].flags & installable_files) {
             add_command(c, INSTALL_CMD,
                         p->entries[i].file,
                         p->entries[i].dst,
+                        tmp,
                         p->entries[i].mode);
         }
 
+        nvfree(tmp);
+
         /*
-         * delete the temporary libGL.la file generated based on
-         * the template earlier.
+         * delete the temporary libGL.la and .desktop files generated
+         * based on templates earlier.
          */
 
-        if (p->entries[i].flags & FILE_TYPE_LIBGL_LA) {
+        if ((p->entries[i].flags & FILE_TYPE_LIBGL_LA) ||
+                (p->entries[i].flags & FILE_TYPE_DOT_DESKTOP)) {
             add_command(c, DELETE_CMD,
                         p->entries[i].file);
         }
+
+        if (op->selinux_enabled &&
+            ((p->entries[i].flags & FILE_TYPE_SHARED_LIB) ||
+             (p->entries[i].flags & FILE_TYPE_XMODULE_SHARED_LIB))) {
+            tmp = nvstrcat(op->utils[CHCON], " -t ", op->selinux_chcon_type,
+                           " ", p->entries[i].dst, NULL);
+            add_command(c, RUN_CMD, tmp);
+            nvfree(tmp);
+        }
     }
 
 
@@ -271,6 +361,7 @@ void free_command_list(Options *op, CommandList *cl)
         c = &cl->cmds[i];
         if (c->s0) free(c->s0);
         if (c->s1) free(c->s1);
+        if (c->s2) free(c->s2);
     }
 
     if (cl->cmds) free(cl->cmds);
@@ -279,7 +370,30 @@ void free_command_list(Options *op, CommandList *cl)
 
 } /* free_command_list() */
 
+/*
+ * execute_run_command() - execute a RUN_CMD from the command list.
+ */
 
+static inline int execute_run_command(Options *op, float percent, const char *cmd)
+{
+    int ret;
+    char *data;
+
+    ui_expert(op, "Executing: %s", cmd);
+    ui_status_update(op, percent, "Executing: `%s` "
+                     "(this may take a moment...)", cmd);
+    ret = run_command(op, cmd, &data, TRUE, 0, TRUE);
+    if (ret != 0) {
+        ui_error(op, "Failed to execute `%s`: %s", cmd, data);
+        ret = continue_after_error(op, "Failed to execute `%s`", cmd);
+        if (!ret) {
+            nvfree(data);
+            return FALSE;
+        }
+    }
+    if (data) free(data);
+    return TRUE;
+} /* execute_run_command() */
 
 /*
  * execute_command_list() - execute the commands in the command list.
@@ -291,7 +405,6 @@ int execute_command_list(Options *op, CommandList *c,
                          const char *title, const char *msg)
 {
     int i, ret;
-    char *data;
     float percent;
 
     ui_status_begin(op, title, msg);
@@ -314,24 +427,23 @@ int execute_command_list(Options *op, CommandList *c,
                                            c->cmds[i].s1);
                 if (!ret) return FALSE;
             } else {
+                /*
+                 * perform post-install step before logging the backup
+                 */
+                if (c->cmds[i].s2 &&
+                    !execute_run_command(op, percent, c->cmds[i].s2)) {
+                    return FALSE;
+                }
+
                 log_install_file(op, c->cmds[i].s1);
                 append_to_rpm_file_list(op, &c->cmds[i]);
             }
             break;
             
         case RUN_CMD:
-            ui_expert(op, "Executing: %s", c->cmds[i].s0);
-            ui_status_update(op, percent, "Executing: `%s` "
-                             "(this may take a moment...)", c->cmds[i].s0);
-            ret = run_command(op, c->cmds[i].s0, &data, TRUE, 0, TRUE);
-            if (ret != 0) {
-                ui_error(op, "Failed to execute `%s`: %s",
-                         c->cmds[i].s0, data);
-                ret = continue_after_error(op, "Failed to execute `%s`",
-                                           c->cmds[i].s0);
-                if (!ret) return FALSE;
+            if (!execute_run_command(op, percent, c->cmds[i].s0)) {
+                return FALSE;
             }
-            if (data) free(data);
             break;
 
         case SYMLINK_CMD:
@@ -395,20 +507,40 @@ int execute_command_list(Options *op, CommandList *c,
 typedef struct {
     const char *name;
     int len;
-} ConflictingLibInfo;
+
+    /*
+     * if requiredString is non-NULL, then a file must have this
+     * string in order to be considered a conflicting file; we use
+     * this to only consider "libglx.*" files conflicts if they have
+     * the string "glxModuleData".
+     */
+
+    const char *requiredString;
+} ConflictingFileInfo;
+
+static void find_conflicting_files(Options *op,
+                                   char *path,
+                                   ConflictingFileInfo *files,
+                                   FileList *l);
 
 static void find_conflicting_libraries(Options *op,
                                        const char *prefix,
-                                       ConflictingLibInfo *libs,
+                                       ConflictingFileInfo *libs,
                                        FileList *l);
 
-static ConflictingLibInfo __xfree86_libs[] = {
-    { "libGLcore.",     10 /* strlen("libGLcore.") */    },
-    { "libGL.",         6  /* strlen("libGL.") */        },
-    { "libGLwrapper.",  13 /* strlen("libGLwrapper.") */ },
-    { "libglx.",        7  /* strlen("libglx.") */       },
-    { "libXvMCNVIDIA",  14 /* strlen("libXvMCNVIDIA") */ },
-    { NULL, 0 }
+static ConflictingFileInfo __xfree86_libs[] = {
+    { "libGLcore.",       10, /* strlen("libGLcore.") */       NULL            },
+    { "libGL.",           6,  /* strlen("libGL.") */           NULL            },
+    { "libGLwrapper.",    13, /* strlen("libGLwrapper.") */    NULL            },
+    { "libglx.",          7,  /* strlen("libglx.") */          "glxModuleData" },
+    { "libXvMCNVIDIA",    13, /* strlen("libXvMCNVIDIA") */    NULL            },
+    { "libnvidia-cfg.",   14, /* strlen("libnvidia-cfg.") */   NULL            },
+    { "nvidia_drv.",      11, /* strlen("nvidia_drv.") */      NULL            },
+    { "libcuda.",         8,  /* strlen("libcuda.") */         NULL            },
+    { "libvdpau.",        9,  /* strlen("libvdpau.") */        NULL            },
+    { "libvdpau_trace.",  15, /* strlen("libvdpau_trace.") */  NULL            },
+    { "libvdpau_nvidia.", 16, /* strlen("libvdpau_nvidia.") */ NULL            },
+    { NULL,               0,                                   NULL            }
 };
 
 /*
@@ -417,19 +549,40 @@ static ConflictingLibInfo __xfree86_libs[] = {
  * libdirs.
  */
 
-void find_conflicting_xfree86_libraries(Options *op,
-                                        const char *xprefix, FileList *l)
+static void find_conflicting_xfree86_libraries(Options *op,
+                                               const char *xprefix,
+                                               FileList *l)
 {
     find_conflicting_libraries(op, xprefix, __xfree86_libs, l);
 
 } /* find_conflicting_xfree86_libraries() */
 
 
-static ConflictingLibInfo __opengl_libs[] = {
-    { "libGLcore.",     10 /* strlen("libGLcore.") */     },
-    { "libGL.",         6  /* strlen("libGL.") */         },
-    { "libnvidia-tls.", 14 /* strlen("libnvidia-tls.") */ },
-    { "libGLwrapper.",  13 /* strlen("libGLwrapper.") */  },
+
+/*
+ * find_conflicting_xfree86_libraries_fullpath() - same as
+ * find_conflicting_xfree86_libraries, but bypasses the
+ * find_conflicting_libraries step, which appends "lib", "lib64", and
+ * "lib32" to the path name.  Use this when you have the fullpath that
+ * you want searched.
+ */
+
+static void find_conflicting_xfree86_libraries_fullpath(Options *op,
+                                                        const char *path,
+                                                        FileList *l)
+{
+    find_conflicting_files(op, (char *) path, __xfree86_libs, l);
+    
+} /* find_conflicting_xfree86_libraries_fullpath() */
+
+
+
+static ConflictingFileInfo __opengl_libs[] = {
+    { "libGLcore.",     10, /* strlen("libGLcore.") */     NULL },
+    { "libGL.",         6,  /* strlen("libGL.") */         NULL },
+    { "libnvidia-tls.", 14, /* strlen("libnvidia-tls.") */ NULL },
+    { "libGLwrapper.",  13, /* strlen("libGLwrapper.") */  NULL },
+    { "libcuda.",       8,  /* strlen("libcuda.") */       NULL },
     { NULL, 0 }
 };
 
@@ -439,8 +592,9 @@ static ConflictingLibInfo __opengl_libs[] = {
  * libdirs.
  */
 
-void find_conflicting_opengl_libraries(Options *op,
-                                       const char *glprefix, FileList *l)
+static void find_conflicting_opengl_libraries(Options *op,
+                                              const char *glprefix,
+                                              FileList *l)
 {
     find_conflicting_libraries(op, glprefix, __opengl_libs, l);
 
@@ -451,41 +605,45 @@ void find_conflicting_opengl_libraries(Options *op,
 /*
  * find_conflicting_kernel_modules() - search for conflicting kernel
  * modules under the kernel module installation prefix.
- *
- * XXX rather than use a fixed list of prefixes, maybe we should scan for the
- * kernel module name anywhere under /lib/modules/`uname -r`/ ?
  */
 
 static void find_conflicting_kernel_modules(Options *op,
                                             Package *p, FileList *l)
 {
     int i, n = 0;
-    char *prefixes[5];
+    ConflictingFileInfo files[2];
+    char *paths[3];
     char *tmp = get_kernel_name(op);
 
-    prefixes[0] = op->kernel_module_installation_path;
+    memset(files, 0, sizeof(files));
+    files[1].name = NULL;
+    files[1].len = 0;
+    paths[0] = op->kernel_module_installation_path;
 
     if (tmp) {
-        prefixes[1] = nvstrcat("/lib/modules/", tmp, "/kernel/drivers/video", NULL);
-        prefixes[2] = nvstrcat("/lib/modules/", tmp, "/kernel/drivers/char", NULL);
-        prefixes[3] = nvstrcat("/lib/modules/", tmp, "/video", NULL);
-        prefixes[4] = NULL;
+        paths[1] = nvstrcat("/lib/modules/", tmp, NULL);
+        paths[2] = NULL;
     } else {
-        prefixes[1] = NULL;
+        paths[1] = NULL;
     }
     
-    /* look for all the conflicting module names in all the possible prefixes */
-    
-    for (i = 0; prefixes[i]; i++) {
+    for (i = 0; paths[i]; i++) {
         for (n = 0; p->bad_module_filenames[n]; n++) {
-            find_matches(prefixes[i], p->bad_module_filenames[n], l, TRUE);
+            /*
+             * Recursively search for this conflicting kernel module
+             * relative to the current prefix.
+             */
+            files[0].name = p->bad_module_filenames[n];
+            files[0].len = strlen(files[0].name);
+
+            find_conflicting_files(op, paths[i], files, l);
         }
     }
 
-    /* free any prefixes we nvstrcat()ed above  */
+    /* free any paths we nvstrcat()'d above  */
 
-    for (i = 1; prefixes[i]; i++) {
-        nvfree(prefixes[i]);
+    for (i = 1; paths[i]; i++) {
+        nvfree(paths[i]);
     }
 
 } /* find_conflicting_kernel_modules() */
@@ -498,7 +656,7 @@ static void find_conflicting_kernel_modules(Options *op,
  * FileList.
  */
 
-static void find_existing_files(Package *p, FileList *l, unsigned int flag)
+static void find_existing_files(Package *p, FileList *l, uint64_t flag)
 {
     int i;
     struct stat stat_buf;
@@ -514,26 +672,105 @@ static void find_existing_files(Package *p, FileList *l, unsigned int flag)
 
 
 
+/*
+ * ignore_conflicting_file() - ignore (i.e., do not put it on the list
+ * of files to backup) the conflicting file 'filename' if requiredString
+ * is non-NULL and we cannot find the string in 'filename'.
+ */
+
+static int ignore_conflicting_file(Options *op,
+                                   const char *filename,
+                                   const char *requiredString)
+{
+    int fd = -1;
+    struct stat stat_buf;
+    char *file = MAP_FAILED;
+    int ret = FALSE;
+    int i, len;
+
+    /* if no requiredString, do not ignore this conflicting file */
+
+    if (!requiredString) return FALSE;
+
+    if ((fd = open(filename, O_RDONLY)) == -1) {
+        ui_error(op, "Unable to open '%s' for reading (%s)",
+                 filename, strerror(errno));
+        goto cleanup;
+    }
+
+    if (fstat(fd, &stat_buf) == -1) {
+        ui_error(op, "Unable to determine size of '%s' (%s)",
+                 filename, strerror(errno));
+        goto cleanup;
+    }
+
+    if (!stat_buf.st_size) {
+        goto cleanup;
+    }
+
+    if ((file = mmap(0, stat_buf.st_size, PROT_READ,
+                     MAP_FILE | MAP_SHARED, fd, 0)) == MAP_FAILED) {
+        ui_error(op, "Unable to map file '%s' for reading (%s)",
+                 filename, strerror(errno));
+        goto cleanup;
+    }
+
+    /*
+     * if the requiredString is not found within the mapping of file,
+     * ignore the conflicting file; when scanning for the string,
+     * ensure that the string is either at the end of the file, or
+     * followed by '\0'.
+     */
+
+    ret = TRUE;
+
+    len = strlen(requiredString);
+
+    for (i = 0; (i + len) <= stat_buf.st_size; i++) {
+        if ((strncmp(&file[i], requiredString, len) == 0) &&
+            (((i + len) == stat_buf.st_size) || (file[i+len] == '\0'))) {
+            ret = FALSE;
+            break;
+        }
+    }
+
+    /* fall through to cleanup */
+
+ cleanup:
+
+    if (file != MAP_FAILED) {
+        munmap(file, stat_buf.st_size);
+    }
+
+    if (fd != -1) {
+        close(fd);
+    }
+
+    return ret;
+
+} /* ignore_conflicting_file() */
+
+
+
 
 /*
- * find_conflicting_libraries() - search for any conflicting
- * libraries in all relevant libdirs within the hierarchy under
+ * find_conflicting_files() - search for any conflicting
+ * files in all the specified paths within the hierarchy under
  * the given prefix.
  */
 
-static void find_conflicting_libraries(Options *op,
-                                       const char *prefix,
-                                       ConflictingLibInfo *libs,
-                                       FileList *l)
+static void find_conflicting_files(Options *op,
+                                   char *path,
+                                   ConflictingFileInfo *files,
+                                   FileList *l)
 {
     int i;
-    char *paths[3];
+    char *paths[2];
     FTS *fts;
     FTSENT *ent;
 
-    paths[0] = nvstrcat(prefix, "/", "lib", NULL);
-    paths[1] = nvstrcat(prefix, "/", "lib64", NULL);
-    paths[2] = NULL;
+    paths[0] = path; /* search root */
+    paths[1] = NULL;
 
     fts = fts_open(paths, FTS_LOGICAL | FTS_NOSTAT, NULL);
     if (!fts) return;
@@ -542,9 +779,12 @@ static void find_conflicting_libraries(Options *op,
         switch (ent->fts_info) {
         case FTS_F:
         case FTS_SLNONE:
-            for (i = 0; libs[i].name; i++) {
-                if (!strncmp(ent->fts_name, libs[i].name, libs[i].len))
+            for (i = 0; files[i].name; i++) {
+                if (!strncmp(ent->fts_name, files[i].name, files[i].len) &&
+                    !ignore_conflicting_file(op, ent->fts_path,
+                                             files[i].requiredString)) {
                     add_file_to_list(NULL, ent->fts_path, l);
+                }
             }
             break;
 
@@ -568,7 +808,54 @@ static void find_conflicting_libraries(Options *op,
 
     fts_close(fts);
 
-    for (i = 0; paths[i]; i++)
+} /* find_conflicting_files() */
+
+
+
+
+/*
+ * find_conflicting_libraries() - search for any conflicting
+ * libraries in all relevant libdirs within the hierarchy under
+ * the given prefix.
+ */
+
+static void find_conflicting_libraries(Options *op,
+                                       const char *prefix,
+                                       ConflictingFileInfo *files,
+                                       FileList *l)
+{
+    int i, j;
+    char *paths[4];
+
+    paths[0] = nvstrcat(prefix, "/", "lib", NULL);
+    paths[1] = nvstrcat(prefix, "/", "lib64", NULL);
+    paths[2] = nvstrcat(prefix, "/", "lib32", NULL);
+    paths[3] = NULL;
+
+    for (i = 0; paths[i]; i++) {
+        for (j = 0; (j < 3) && paths[i]; j++) {
+            /*
+             * XXX Check if any one of the 'paths' entries really
+             * is a symbolic link pointing to one of the other
+             * entries. The logic could be made smarter, since it's
+             * unlikely that ../lib32 would be a symbolic link to
+             * ../lib64 or vice versa.
+             */
+            if (!paths[j] || (i == j)) continue;
+
+            if (is_symbolic_link_to(paths[i], paths[j])) {
+                ui_expert(op, "The conflicting library search path "
+                          "'%s' is a symbolic link to the library "
+                          "search path '%s'; skipping '%s'.",
+                          paths[i], paths[j], paths[i]);
+                free(paths[i]); paths[i] = NULL;
+            }
+        }
+
+        if (paths[i]) find_conflicting_files(op, paths[i], files, l);
+    }
+
+    for (i = 0; i < 3; i++)
         nvfree(paths[i]);
 
 } /* find_conflicting_libraries() */
@@ -580,27 +867,75 @@ static void find_conflicting_libraries(Options *op,
  * brute-force algorithm.
  */
 
-void condense_file_list(FileList *l)
+static void condense_file_list(Package *p, FileList *l)
 {
     char **s = NULL;
-    int n = 0, i, j, match;
+    int n = 0, i, j, keep;
+
+    struct stat stat_buf, *stat_bufs;
+
+    /* allocate enough space in our temporary 'stat' array */
+
+    if (l->num) {
+        stat_bufs  = nvalloc(sizeof(struct stat) * l->num);
+    } else {
+        stat_bufs  = NULL;
+    }
+    
+    /*
+     * walk through our original (uncondensed) list of files and move
+     * unique files to a new (condensed) list.  For each file in the
+     * original list, get the filesystem information for the file, and
+     * then compare that to the filesystem information for all the
+     * files in the new list.  If the file from the original list does
+     * not match any file in the new list, add it to the new list.
+     */
 
     for (i = 0; i < l->num; i++) {
-        match = FALSE;
-        for (j = 0; j < n; j++) {
-            if (strcmp(l->filename[i], s[j]) == 0) {
-                match = TRUE;
+        keep = TRUE;
+
+        if (lstat(l->filename[i], &stat_buf) == -1)
+            continue;
+
+        /*
+         * check if this file is in the package we're trying to
+         * install; we don't want to remove files that are in the
+         * package; symlinks may have tricked us into looking for
+         * conflicting files inside our unpacked .run file.
+         */
+
+        for (j = 0; j < p->num_entries; j++) {
+            if ((p->entries[j].device == stat_buf.st_dev) &&
+                (p->entries[j].inode == stat_buf.st_ino)) {
+                keep = FALSE;
                 break;
             }
         }
-        
-        if (!match) {
+
+        for (j = 0; keep && (j < n); j++) {
+
+            /*
+             * determine if the two files are the same by comparing
+             * device and inode
+             */
+
+            if ((stat_buf.st_dev == stat_bufs[j].st_dev) &&
+                (stat_buf.st_ino == stat_bufs[j].st_ino)) {
+                keep = FALSE;
+                break;
+            }
+        }
+
+        if (keep) {
             s = (char **) nvrealloc(s, sizeof(char *) * (n + 1));
             s[n] = nvstrdup(l->filename[i]);
+            stat_bufs[n] = stat_buf;
             n++;
         }
     }
     
+    if (stat_bufs) nvfree((void *)stat_bufs);
+
     for (i = 0; i < l->num; i++) free(l->filename[i]);
     free(l->filename);
 
@@ -627,6 +962,7 @@ static void add_command(CommandList *c, int cmd, ...)
     c->cmds[n].cmd  = cmd;
     c->cmds[n].s0   = NULL;
     c->cmds[n].s1   = NULL;
+    c->cmds[n].s2   = NULL;
     c->cmds[n].mode = 0x0;
     
     va_start(ap, cmd);
@@ -637,6 +973,8 @@ static void add_command(CommandList *c, int cmd, ...)
         c->cmds[n].s0 = nvstrdup(s);
         s = va_arg(ap, char *);
         c->cmds[n].s1 = nvstrdup(s);
+        s = va_arg(ap, char *);
+        c->cmds[n].s2 = nvstrdup(s);
         c->cmds[n].mode = va_arg(ap, mode_t);
         break;
       case BACKUP_CMD:
@@ -669,50 +1007,6 @@ static void add_command(CommandList *c, int cmd, ...)
 
 
 
-/*
- * find_matches() - given a directory, a filename, and an existing
- * FileList data structure, open the specified directory, and look for
- * any entries that match the filename.
- *
- * If the parameter 'exact' is TRUE, then the filenames must match
- * exactly.  If 'exact' is FALSE, then only the beginning of the
- * directory entry name must match the filename in question.
- *
- * This could alternatively be implemented using glob(3).
- */
-
-static void find_matches(const char *directory, const char *filename,
-                         FileList *l, const int exact)
-{
-    struct stat stat_buf;
-    struct dirent *ent;
-    int len;
-    DIR *dir;
-    
-    if (lstat(directory, &stat_buf) == -1) return;
-    if (S_ISDIR(stat_buf.st_mode) == 0) return;
-    if ((dir = opendir(directory)) == NULL) return;
-
-    len = strlen(filename);
-
-    while ((ent = readdir(dir)) != NULL) {
-        if (exact) {
-            if (strcmp(ent->d_name, filename) == 0) {
-                add_file_to_list(directory, ent->d_name, l);
-            }
-        } else {
-            if (strncmp(ent->d_name, filename, len) == 0) {
-                add_file_to_list(directory, ent->d_name, l);
-            }
-        }
-    }
-    
-    closedir (dir); 
-    
-} /* find_matches() */
-
-
-
 /*
  * add_file_to_list() - concatenate the given directory and filename,
  * appending to the FileList structure.  If the 'directory' parameter
