@@ -54,8 +54,7 @@
 
 
 static Package *parse_manifest(Options *op);
-
-
+static int install_kernel_module(Options *op,  Package *p);
 
 
 /*
@@ -79,11 +78,12 @@ int install_from_cwd(Options *op)
     Package *p;
     CommandList *c;
     const char *msg;
+    int ran_pre_install_hook = FALSE;
 
     static const char edit_your_xf86config[] =
         "Please update your XF86Config or xorg.conf file as "
         "appropriate; see the file /usr/share/doc/"
-        "NVIDIA_GLX-1.0/README.txt for details.";
+        "NVIDIA_GLX-1.0/README for details.";
 
     static const char suse_edit_your_xf86config[] =
         "On SuSE Linux/United Linux please use SaX2 now to enable "
@@ -96,106 +96,53 @@ int install_from_cwd(Options *op)
     
     if ((p = parse_manifest(op)) == NULL) goto failed;
     
-    ui_set_title(op, "%s (%d.%d-%d)", p->description,
-                 p->major, p->minor, p->patch);
-
-    /* make sure the kernel module is unloaded */
-    
-    if (!check_for_unloaded_kernel_module(op, p)) goto failed;
+    ui_set_title(op, "%s (%s)", p->description, p->version);
     
     /* check that we are not running any X server */
 
     if (!check_for_running_x(op)) goto failed;
 
+    /* make sure the kernel module is unloaded */
+    
+    if (!check_for_unloaded_kernel_module(op, p)) goto failed;
+    
     /* ask the user to accept the license */
     
     if (!get_license_acceptance(op)) return FALSE;
     
+    ui_log(op, "Installing NVIDIA driver version %s.", p->version);
+
     /*
      * determine the current NVIDIA version (if any); ask the user if
      * they really want to overwrite the existing installation
      */
 
-    /* print warning message if a legacy gpu is detected in the system */
-    
-    check_for_legacy_gpu(op, p);
-
     if (!check_for_existing_driver(op, p)) return FALSE;
-    
-    /* determine where to install the kernel module */
-    
-    if (!determine_kernel_module_installation_path(op)) goto failed;
 
-    /* check '/proc/sys/kernel/modprobe' */
+    /* run the distro preinstall hook */
 
-    if (!check_proc_modprobe_path(op)) return FALSE;
-
-    /*
-     * do nvchooser-style logic to decide if we have a prebuilt kernel
-     * module for their kernel
-     *
-     * XXX One could make the argument that we should not actually do
-     * the building/linking now, but just add this to the list of
-     * operations and do it when we execute the operation list.  I
-     * think it's better to make sure we have a kernel module early on
-     * -- a common problem for users will be not having a prebuilt
-     * kernel interface for their kernel, and not having the kernel
-     * headers installed, so it's better to catch that earlier on.
-     */
-    
-    if (find_precompiled_kernel_interface(op, p)) {
-
-        /*
-         * we have a prebuild kernel interface, so now link the kernel
-         * interface with the binary portion of the kernel module.
-         *
-         * XXX if linking fails, maybe we should fall through and
-         * attempt to build the kernel module?  No, if linking fails,
-         * then there is something pretty seriously wrong... better to
-         * abort.
-         */
-        
-        if (!link_kernel_module(op, p)) goto failed;
-
-    } else {
-        /*
-         * make sure the required development tools are present on
-         * this system before attempting to verify the compiler and
-         * trying to build a custom kernel interface.
-         */
-        if (!check_development_tools(op)) goto failed;
-
-        /*
-         * we do not have a prebuilt kernel interface; thus we'll need
-         * to compile the kernel interface, so determine where the
-         * kernel header files are
-         */
-        
-        if (!determine_kernel_source_path(op)) goto failed;
-        determine_kernel_output_path(op);
-    
-        /*
-         * make sure that the selected or default system compiler
-         * is compatible with the target kernel; the user may choose
-         * to override the check.
-         */
-        if (!check_cc_version(op, p)) goto failed;
-
-        /* and now, build the kernel interface */
-        
-        if (!build_kernel_module(op, p)) goto failed;
+    if (!run_distro_hook(op, "pre-install")) {
+        if (!ui_yes_no(op, TRUE,
+                       "The distribution-provided pre-install script failed!  "
+                       "Continue installation anyway?")) {
+            goto failed;
+        }
     }
+    ran_pre_install_hook = TRUE;
 
-    /*
-     * if we got this far, we have a complete kernel module; test it
-     * to be sure it's OK
-     */
-    
-    if (!test_kernel_module(op, p)) goto failed;
-    
-    /* add the kernel module to the list of things to install */
-    
-    if (!add_kernel_module_to_package(op, p)) goto failed;
+    /* attempt to build a kernel module for the target kernel */
+
+    if (!op->no_kernel_module) {
+        if (!install_kernel_module(op, p)) goto failed;
+    } else {
+        ui_warn(op, "You specified the '--no-kernel-module' command line "
+                "option, nvidia-installer will not install a kernel "
+                "module as part of this driver installation, and it will "
+                "not remove existing NVIDIA kernel modules not part of "
+                "an earlier NVIDIA driver installation.  Please ensure "
+                "that an NVIDIA kernel module matching this driver version "
+                "is installed seperately.");
+    }
     
     /*
      * if we are only installing the kernel module, then remove
@@ -223,13 +170,14 @@ int install_from_cwd(Options *op)
         select_tls_class(op, p);
 
         /*
-         * if the package contains any libGL.la files, process them
-         * (perform some search and replacing so that they reflect the
-         * correct installation path, etc) and add them to the
-         * packagelist to be installed.
+         * if the package contains any libGL.la or .desktop files,
+         * process them (perform some search and replacing so
+         * that they reflect the correct installation path, etc)
+         * and add them to the package list (files to be installed).
          */
         
         process_libGL_la_files(op, p);
+        process_dot_desktop_files(op, p);
 
 #if defined(NV_X86_64)
         /*
@@ -268,7 +216,7 @@ int install_from_cwd(Options *op)
 
     /* call the ui to get approval for the list of commands */
     
-    if (!ui_approve_command_list(op, c, p->description)) return FALSE;
+    if (!ui_approve_command_list(op, c, "%s", p->description)) return FALSE;
     
     /* initialize the backup log file */
 
@@ -280,6 +228,10 @@ int install_from_cwd(Options *op)
 
     if (!do_install(op, p, c)) goto failed;
 
+    /* run the distro postinstall script */
+
+    run_distro_hook(op, "post-install");
+
     /* 
      * XXX IMPLEMENT ME: generate an XF86Config file?
      */
@@ -305,10 +257,10 @@ int install_from_cwd(Options *op)
     if (op->kernel_module_only) {
         ui_message(op, "Installation of the kernel module for the %s "
                    "(version %s) is now complete.",
-                   p->description, p->version_string);
+                   p->description, p->version);
     } else {
         ui_message(op, "Installation of the %s (version: %s) is now "
-                   "complete.  %s", p->description, p->version_string, msg);
+                   "complete.  %s", p->description, p->version, msg);
     }
     
     return TRUE;
@@ -326,13 +278,107 @@ int install_from_cwd(Options *op)
                  "on fixing installation problems in the README available "
                  "on the Linux driver download page at www.nvidia.com.");
     }
-    
+
+    if (ran_pre_install_hook)
+        run_distro_hook(op, "failed-install");
+
     return FALSE;
 
 } /* install_from_cwd() */
 
 
 
+/*
+ * install_kernel_module() - attempt to build and install a kernel
+ * module for the running kernel; we first check if a prebuilt kernel
+ * interface file exists. If yes, we try to link it into the final
+ * kernel module, else we try to build one from source.
+ *
+ * If we succeed in building a kernel module, we attempt to load it
+ * into the host kernel and add it to the list of files to install if
+ * the load attempt succeeds.
+ */
+
+static int install_kernel_module(Options *op,  Package *p)
+{
+    /* determine where to install the kernel module */
+    
+    if (!determine_kernel_module_installation_path(op)) return FALSE;
+
+    /* check '/proc/sys/kernel/modprobe' */
+
+    if (!check_proc_modprobe_path(op)) return FALSE;
+
+    /*
+     * do nvchooser-style logic to decide if we have a prebuilt kernel
+     * module for their kernel
+     *
+     * XXX One could make the argument that we should not actually do
+     * the building/linking now, but just add this to the list of
+     * operations and do it when we execute the operation list.  I
+     * think it's better to make sure we have a kernel module early on
+     * -- a common problem for users will be not having a prebuilt
+     * kernel interface for their kernel, and not having the kernel
+     * headers installed, so it's better to catch that earlier on.
+     */
+    
+    if (find_precompiled_kernel_interface(op, p)) {
+
+        /*
+         * we have a prebuild kernel interface, so now link the kernel
+         * interface with the binary portion of the kernel module.
+         *
+         * XXX if linking fails, maybe we should fall through and
+         * attempt to build the kernel module?  No, if linking fails,
+         * then there is something pretty seriously wrong... better to
+         * abort.
+         */
+        
+        if (!link_kernel_module(op, p)) return FALSE;
+
+    } else {
+        /*
+         * make sure the required development tools are present on
+         * this system before attempting to verify the compiler and
+         * trying to build a custom kernel interface.
+         */
+        if (!check_development_tools(op, p)) return FALSE;
+
+        /*
+         * make sure that the selected or default system compiler
+         * is compatible with the target kernel; the user may choose
+         * to override the check.
+         */
+        if (!check_cc_version(op, p)) return FALSE;
+
+        /*
+         * we do not have a prebuilt kernel interface; thus we'll need
+         * to compile the kernel interface, so determine where the
+         * kernel source files are.
+         */
+        
+        if (!determine_kernel_source_path(op, p)) return FALSE;
+    
+        /* and now, build the kernel interface */
+        
+        if (!build_kernel_module(op, p)) return FALSE;
+    }
+
+    /*
+     * if we got this far, we have a complete kernel module; test it
+     * to be sure it's OK
+     */
+    
+    if (!test_kernel_module(op, p)) return FALSE;
+    
+    /* add the kernel module to the list of things to install */
+    
+    if (!add_kernel_module_to_package(op, p)) return FALSE;
+    
+    return TRUE;
+}
+
+
 /*
  * add_this_kernel() - build a precompiled kernel interface for the
  * running kernel, and repackage the .run file to include the new
@@ -349,7 +395,7 @@ int add_this_kernel(Options *op)
 
     /* find the kernel header files */
 
-    if (!determine_kernel_source_path(op)) goto failed;
+    if (!determine_kernel_source_path(op, p)) goto failed;
 
     /* build the precompiled kernel interface */
 
@@ -379,7 +425,7 @@ int add_this_kernel(Options *op)
  * The first nine lines of the .manifest file are:
  *
  *   - a description string
- *   - a version string of the form "major.minor-patch"
+ *   - a version string
  *   - the kernel module file name
  *   - the kernel interface file name
  *   - the kernel module name (what `rmmod` and `modprobe` should use)
@@ -407,7 +453,7 @@ static Package *parse_manifest (Options *op)
     char *buf, *c, *flag , *tmpstr;
     int done, n, line;
     int fd, len = 0;
-    struct stat stat_buf;
+    struct stat stat_buf, entry_stat_buf;
     Package *p;
     char *manifest = MAP_FAILED, *ptr;
     
@@ -432,27 +478,25 @@ static Package *parse_manifest (Options *op)
     /* the first line is the description */
 
     line = 1;
-    p->description = get_next_line(manifest, &ptr);
+    p->description = get_next_line(manifest, &ptr, manifest, len);
     if (!p->description) goto invalid_manifest_file;
     
     /* the second line is the version */
     
     line++;
-    p->version_string = get_next_line(ptr, &ptr);
-    if (!p->version_string) goto invalid_manifest_file;
-    if (!nvid_version(p->version_string, &p->major, &p->minor, &p->patch))
-        goto invalid_manifest_file;
+    p->version = get_next_line(ptr, &ptr, manifest, len);
+    if (!p->version) goto invalid_manifest_file;
     
     /* new third line is the kernel interface filename */
 
     line++;
-    p->kernel_interface_filename = get_next_line(ptr, &ptr);
+    p->kernel_interface_filename = get_next_line(ptr, &ptr, manifest, len);
     if (!p->kernel_interface_filename) goto invalid_manifest_file;
 
     /* the fourth line is the kernel module name */
 
     line++;
-    p->kernel_module_name = get_next_line(ptr, &ptr);
+    p->kernel_module_name = get_next_line(ptr, &ptr, manifest, len);
     if (!p->kernel_module_name) goto invalid_manifest_file;
 
     /*
@@ -461,7 +505,7 @@ static Package *parse_manifest (Options *op)
      */
 
     line++;
-    tmpstr = get_next_line(ptr, &ptr);
+    tmpstr = get_next_line(ptr, &ptr, manifest, len);
     if (!tmpstr) goto invalid_manifest_file;
 
     p->bad_modules = NULL;
@@ -482,7 +526,7 @@ static Package *parse_manifest (Options *op)
      */
 
     line++;
-    tmpstr = get_next_line(ptr, &ptr);
+    tmpstr = get_next_line(ptr, &ptr, manifest, len);
     if (!tmpstr) goto invalid_manifest_file;
 
     p->bad_module_filenames = NULL;
@@ -499,7 +543,7 @@ static Package *parse_manifest (Options *op)
     /* the seventh line is the kernel module build directory */
 
     line++;
-    p->kernel_module_build_directory = get_next_line(ptr, &ptr);
+    p->kernel_module_build_directory = get_next_line(ptr, &ptr, manifest, len);
     if (!p->kernel_module_build_directory) goto invalid_manifest_file;
     remove_trailing_slashes(p->kernel_module_build_directory);
 
@@ -509,7 +553,8 @@ static Package *parse_manifest (Options *op)
      */
 
     line++;
-    p->precompiled_kernel_interface_directory = get_next_line(ptr, &ptr);
+    p->precompiled_kernel_interface_directory =
+        get_next_line(ptr, &ptr, manifest, len);
     if (!p->precompiled_kernel_interface_directory)
         goto invalid_manifest_file;
     remove_trailing_slashes(p->precompiled_kernel_interface_directory);
@@ -520,7 +565,7 @@ static Package *parse_manifest (Options *op)
     line++;
     
     do {
-        buf = get_next_line(ptr, &ptr);
+        buf = get_next_line(ptr, &ptr, manifest, len);
         if ((!buf) || (buf[0] == '\0')) {
             done = TRUE;
         } else {
@@ -572,22 +617,34 @@ static Package *parse_manifest (Options *op)
                 p->entries[n].flags |= FILE_TYPE_OPENGL_LIB;
             else if (strcmp(flag, "LIBGL_LA") == 0)
                 p->entries[n].flags |= FILE_TYPE_LIBGL_LA;
-            else if (strcmp(flag, "XFREE86_LIB") == 0)
-                p->entries[n].flags |= FILE_TYPE_XFREE86_LIB;
+            else if (strcmp(flag, "XLIB_STATIC_LIB") == 0)
+                p->entries[n].flags |= FILE_TYPE_XLIB_STATIC_LIB;
+            else if (strcmp(flag, "XLIB_SHARED_LIB") == 0)
+                p->entries[n].flags |= FILE_TYPE_XLIB_SHARED_LIB;
             else if (strcmp(flag, "TLS_LIB") == 0)
                 p->entries[n].flags |= FILE_TYPE_TLS_LIB;
             else if (strcmp(flag, "DOCUMENTATION") == 0)
                 p->entries[n].flags |= FILE_TYPE_DOCUMENTATION;
+            else if (strcmp(flag, "MANPAGE") == 0)
+                p->entries[n].flags |= FILE_TYPE_MANPAGE;
             else if (strcmp(flag, "OPENGL_SYMLINK") == 0)
                 p->entries[n].flags |= FILE_TYPE_OPENGL_SYMLINK;
-            else if (strcmp(flag, "XFREE86_SYMLINK") == 0)
-                p->entries[n].flags |= FILE_TYPE_XFREE86_SYMLINK;
+            else if (strcmp(flag, "XLIB_SYMLINK") == 0)
+                p->entries[n].flags |= FILE_TYPE_XLIB_SYMLINK;
             else if (strcmp(flag, "TLS_SYMLINK") == 0)
                 p->entries[n].flags |= FILE_TYPE_TLS_SYMLINK;
             else if (strcmp(flag, "INSTALLER_BINARY") == 0)
                 p->entries[n].flags |= FILE_TYPE_INSTALLER_BINARY;
             else if (strcmp(flag, "UTILITY_BINARY") == 0)
                 p->entries[n].flags |= FILE_TYPE_UTILITY_BINARY;
+            else if (strcmp(flag, "UTILITY_BIN_SYMLINK") == 0)
+                p->entries[n].flags |= FILE_TYPE_UTILITY_BIN_SYMLINK;
+            else if (strcmp(flag, "DOT_DESKTOP") == 0)
+                p->entries[n].flags |= FILE_TYPE_DOT_DESKTOP;
+            else if (strcmp(flag, "XMODULE_SHARED_LIB") == 0)
+                p->entries[n].flags |= FILE_TYPE_XMODULE_SHARED_LIB;
+            else if (strcmp(flag, "XMODULE_SYMLINK") == 0)
+                p->entries[n].flags |= FILE_TYPE_XMODULE_SYMLINK;
             else {
                 nvfree(flag);
                 goto invalid_manifest_file;
@@ -659,7 +716,21 @@ static Package *parse_manifest (Options *op)
             if (p->entries[n].name) p->entries[n].name++;
             
             if (!p->entries[n].name) p->entries[n].name = p->entries[n].file;
-            
+
+            /*
+             * store the inode and device information, so that we can
+             * later recognize it, to avoid accidentally moving it as
+             * part of the 'find_conflicting_files' path
+             */
+
+            if (stat(p->entries[n].file, &entry_stat_buf) != -1) {
+                p->entries[n].inode = entry_stat_buf.st_ino;
+                p->entries[n].device = entry_stat_buf.st_dev;
+            } else {
+                p->entries[n].inode = 0;
+                p->entries[n].device = 0;
+            }
+
             /* free the line */
             
             free(buf);
@@ -692,3 +763,48 @@ static Package *parse_manifest (Options *op)
     return NULL;
        
 } /* parse_manifest() */
+
+
+
+/*
+ * add_package_entry() - add a PackageEntry to the package's entries
+ * array.
+ */
+
+void add_package_entry(Package *p,
+                       char *file,
+                       char *path,
+                       char *name,
+                       char *target,
+                       char *dst,
+                       uint64_t flags,
+                       mode_t mode)
+{
+    int n;
+    struct stat stat_buf;
+
+    n = p->num_entries;
+
+    p->entries =
+        (PackageEntry *) nvrealloc(p->entries, (n + 1) * sizeof(PackageEntry));
+
+    p->entries[n].file   = file;
+    p->entries[n].path   = path;
+    p->entries[n].name   = name;
+    p->entries[n].target = target;
+    p->entries[n].dst    = dst;
+    p->entries[n].flags  = flags;
+    p->entries[n].mode   = mode;
+
+    if (stat(p->entries[n].file, &stat_buf) != -1) {
+        p->entries[n].inode = stat_buf.st_ino;
+        p->entries[n].device = stat_buf.st_dev;
+    } else {
+        p->entries[n].inode = 0;
+        p->entries[n].device = 0;
+    }
+
+    p->num_entries++;
+
+} /* add_package_entry() */
+
