@@ -68,6 +68,12 @@ int snarf(Options *op, const char *url, int out_fd, uint32 flags)
     UrlResource *rsrc = NULL;
     int ret;
     
+    if (op->no_network) {
+        ui_error(op, "Unable to access file '%s', because the '--no-network' "
+                 "commandline option was specified.", url);
+        return FALSE;
+    }
+
     rsrc = url_resource_new();
     rsrc->url = url_new();
     rsrc->op = op;
@@ -231,7 +237,7 @@ int dump_data(UrlResource *rsrc, int sock)
         ui_status_begin(rsrc->op, msg, "Downloading");
     }
     
-    while ((bytes_read = read(sock, buf, SNARF_BUFSIZE))) {
+    while ((bytes_read = read(sock, buf, SNARF_BUFSIZE)) > 0) {
 
         if (rsrc->flags & SNARF_FLAGS_STATUS_BAR) {
             total_bytes_read += bytes_read;
